@@ -10,6 +10,10 @@ from graph_kmer_index import KmerIndex
 from shared_memory_wrapper import from_shared_memory, to_shared_memory, SingleSharedArray
 from pathos.multiprocessing import Pool
 from itertools import repeat
+from graph_kmer_index.collision_free_kmer_index import MinimalKmerIndex
+from .parser import OneLineFastaParser, Sequences, OneLineFastaParser2bit, BufferedNumpyParser, OneLineFastaBuffer2Bit
+from .kmers import KmerHash, TwoBitHash
+
 
 def get_reads_as_matrices(read_file_name, chunk_size=500000, max_read_length=150):
     return (chunk for chunk in read_fasta_into_chunks(read_file_name, chunk_size, max_read_length))
@@ -74,7 +78,7 @@ def get_kmers_from_read_matrix(read_matrix, mask, k=31, return_only_kmers=False,
     logging.info("k=%d" % k)
     t = time.time()
     numeric_reads = convert_read_matrix_to_numeric(read_matrix)
-    numeric_reads_complement = convert_read_matrix_to_numeric(read_matrix, True)
+    numeric_reads_complement = (numeric_reads + 2) % 4  #convert_read_matrix_to_numeric(read_matrix, True)
     logging.info("Time to convert reads to numeric: %.3f" % (time.time()-t))
     t = time.time()
 
@@ -112,6 +116,56 @@ def get_kmers_from_fasta(fasta_file_name, chunk_size=500000, k=31, max_read_leng
     return get_kmers_from_read_matrix(reads, mask, k=k, return_only_kmers=return_only_kmers)
 
 
+
+def map_fasta_single_thread_with_numpy_parsing(data):
+    logging.info("---------------------")
+    logging.info("---------------------")
+    logging.info("Starting chunk")
+    time_start = time.perf_counter()
+    reads, args = data
+    logging.info("Reads: %s" % reads)
+    cls = KmerIndex
+    if "minimal" in args.kmer_index:
+        cls = MinimalKmerIndex
+
+    if args.no_shared_memory:
+        kmer_index = cls.from_file(args.kmer_index)
+    else:
+        kmer_index = from_shared_memory(cls, "kmer_index"+args.random_id)
+
+    logging.info("Reading from shared memory")
+    #sequence_chunk = from_shared_memory(Sequences, reads)
+    raw_chunk = from_shared_memory(args.buffer_type, reads)
+    logging.info("Parsing sequence chunk")
+    sequence_chunk = raw_chunk.get_sequences()
+    logging.info("Done parsing sequence chunk")
+
+    logging.info("Reading node counts from shared memory")
+    node_counts = from_shared_memory(SingleSharedArray, "counts_shared"+args.random_id).array
+    logging.info("Done reading from shared memory")
+    t = time.perf_counter()
+    if args.use_two_bit_parsing:
+        hashes = TwoBitHash(k=args.kmer_size).get_kmer_hashes(sequence_chunk)
+        logging.info("time to get %d kmer hashes using new numpy: %.3f" % (len(hashes), time.perf_counter() - t))
+        t = time.perf_counter()
+        node_counts += map_kmers_to_graph_index(kmer_index, args.n_nodes, hashes, args.max_hits_per_kmer)
+        logging.info("Done mapping to kmer index. Took %.3f sec" % (time.perf_counter() - t))
+    else:
+        hashes, reverse_complement_hashes, mask = KmerHash(k=args.kmer_size).get_kmer_hashes(sequence_chunk, args.include_reverse_complement)
+        logging.info("time to get %d kmer hashes using new numpy: %.3f" % (len(hashes), time.perf_counter()-t))
+        for h in (hashes, reverse_complement_hashes):
+            if h is None:
+                logging.info("Skipping hashing (reverse complement)")
+                continue
+            h = h[mask]
+            t = time.perf_counter()
+            node_counts += map_kmers_to_graph_index(kmer_index, args.n_nodes, h, args.max_hits_per_kmer)
+            logging.info("Done mapping to kmer index. Took %.3f sec" % (time.perf_counter()-t))
+
+    logging.info("Done with chunk. Took %.3f sec" % (time.perf_counter()-time_start))
+
+
+
 def map_fasta_single_thread(data):
     start_time = time.time()
     reads, args = data
@@ -131,9 +185,27 @@ def map_fasta_single_thread(data):
 
     shared_counts = from_shared_memory(SingleSharedArray, "counts_shared"+args.random_id).array
 
-    index = from_shared_memory(KmerIndex, "kmer_index"+args.random_id)
-    kmers = get_kmers_from_read_matrix(read_matrix, mask, args.kmer_size, True, not args.ignore_reverse_complement)
-    node_counts = map_kmers_to_graph_index(index, args.n_nodes, kmers, args.max_hits_per_kmer)
+    cls = KmerIndex
+    if "minimal" in args.kmer_index:
+        cls = MinimalKmerIndex
+
+    if args.no_shared_memory:
+        logging.info("reading kmer index from file in thread")
+        index = cls.from_file(args.kmer_index)
+    else:
+        index = from_shared_memory(cls, "kmer_index"+args.random_id)
+
+    kmers = get_kmers_from_read_matrix(read_matrix, mask, args.kmer_size, True, args.include_reverse_complement)
+
+
+    t = time.perf_counter()
+    if args.use_numpy:
+        node_counts = index.get_node_counts(kmers)
+    else:
+        node_counts = map_kmers_to_graph_index(index, args.n_nodes, kmers, args.max_hits_per_kmer)
+
+    logging.info("----- Time spent getting node counts: %.4f" % (time.perf_counter()-t))
+
     shared_counts += node_counts
     #shared_memory_name = "node_counts"+str(np.random.randint(0,10e15))
     #to_shared_memory(SingleSharedArray(node_counts), shared_memory_name)
@@ -142,12 +214,19 @@ def map_fasta_single_thread(data):
     #return shared_memory_name
 
 
-def map_fasta(args):
-    logging.info("Mapping fasta to kmer index %s ..." % args.kmer_index)
-    index = args.kmer_index
+def map_fasta(args, kmer_index):
+    logging.info("Mapping fasta to kmer index %s" % args.kmer_index)
     args.random_id = str(np.random.random())
-    to_shared_memory(index, "kmer_index"+args.random_id)
-    n_nodes = index.max_node_id()
+    n_nodes = kmer_index.max_node_id()
+    if not args.no_shared_memory:
+        logging.info("Putting kmer index in shared memory")
+        to_shared_memory(kmer_index, "kmer_index"+args.random_id)
+    else:
+        logging.info("Not putting kmer index in shared memory")
+
+    kmer_index = None
+
+
     args.n_nodes = n_nodes
     start_time = time.time()
 
@@ -156,10 +235,26 @@ def map_fasta(args):
 
     to_shared_memory(SingleSharedArray(node_counts), "counts_shared" + args.random_id)
 
-    reads = read_fasta_into_chunks(args.fasta_file, args.chunk_size, args.max_read_length, write_to_shared_memory=True, process_reads=False)
+    if args.use_two_bit_parsing:
+        logging.info("Using two bit fasta parser")
+        parser = BufferedNumpyParser.from_filename(args.fasta_file, args.chunk_size * 130)
+        args.buffer_type = OneLineFastaBuffer2Bit
+        chunks = parser.get_chunks()
+        reads = (to_shared_memory(chunk) for chunk in chunks)
+        func = map_fasta_single_thread_with_numpy_parsing
+    elif not args.use_cython_file_reading:
+        logging.info("Using numpy fasta parser")
+        fasta_parser = OneLineFastaParser(args.fasta_file, args.chunk_size * 130)
+        reads = fasta_parser.parse(as_shared_memory_object=True)
+        func = map_fasta_single_thread_with_numpy_parsing
+    else:
+        reads = read_fasta_into_chunks(args.fasta_file, args.chunk_size, args.max_read_length, write_to_shared_memory=True, process_reads=False)
+        func = map_fasta_single_thread
 
+    # set index to None, so that it won't be pickled and transferred to processes
     logging.info("Got reads, starting processes")
-    for result in pool.imap(map_fasta_single_thread, zip(reads, repeat(args))):
+    logging.info("Args is %s" % str(args))
+    for result in pool.imap(func, zip(reads, repeat(args))):
         continue
 
     node_counts = from_shared_memory(SingleSharedArray, "counts_shared"+args.random_id).array
